package thermo;

/**
 * <p>Title: JThermo</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: www.LancOS.com</p>
 * @author Claudio Lanconelli
 * @version 1.0
 */

public class Ds1621 {

	protected final static int BASE_ADDRESS = 0x90;   // Temperature I2CBus address
	protected final static int BUFFER_SIZE = 256;

	public final static int E2P_TIMEOUT = -10;

	// Configuration register bits
	protected final static int DONE_BIT = 7;      // Conversion done bit (0 = conversion in progress)
	protected final static int THF_BIT = 6;       // Temperature high flag
	protected final static int TLF_BIT = 5;       // Temperature low flag
	protected final static int NVB_BIT = 4;       // EEPROM busy flag (1 = write in progress)
	protected final static int R1_BIT	= 3;        // reserved bit
	protected final static int R0_BIT = 2;
	protected final static int POL_BIT = 1;       // Output polarity (1 = active high)
	protected final static int ONESHOT_BIT = 0;   // One shot mode (0 = free running)

	protected final static int CmdStartConversion = 0xEE;
	protected final static int CmdStopConversion = 0x22;
	protected final static int CmdAccessConfig = 0xAC;
	protected final static int CmdAccessTH = 0xA1;
	protected final static int CmdAccessTL = 0xA2;
	protected final static int CmdReadTemp = 0xAA;
	protected final static int CmdReadCounter = 0xA8;
	protected final static int CmdReadSlope = 0xA9;

	// DS1631 (extended commands)

	protected final static int CmdStartConversionExt = 0x51;
	protected final static int CmdSoftarePOR = 0x54;

	private boolean one_shot_mode;
	private boolean ds1631_mode;
	private int error_code;
	private int i2c_address;
	private int i2c_buffer[];
	private I2CBus i2c;

    public Ds1621() {
		i2c = new I2CBus();

		one_shot_mode = false;
		ds1631_mode = false;
		error_code = 0;
		i2c_address = BASE_ADDRESS;
	//	i2c.setDelay(delay);
		i2c_buffer = new int[BUFFER_SIZE];
    }

	public void setDS1631Mode(boolean val)
	{
		ds1631_mode = val;
	}

	public void setOneShotMode(boolean val)
	{
		one_shot_mode = val;
	}
	public boolean getOneShotMode()
	{
		return one_shot_mode;
	}

	public int getAddress()
	{
		return (i2c_address - BASE_ADDRESS) / 2 + 1;
	}

	public void setAddress(int addr)
	{
		if (addr >= 1 && addr <= 8)
			i2c_address = BASE_ADDRESS + (addr - 1) * 2;
	}

	public int getError()
	{
		return error_code;
	}

	// Read packet: Start + Address(W) + Command + RepStart + Address(R) + N Bytes + Stop
	private void i2c_read_packet(int len)
	{
		error_code = 0;
		if ( i2c.StartWrite(i2c_address, i2c_buffer, 1) != 1 )
		{
			error_code = i2c.getErrorNumber();
			i2c.Stop();
		}
		else
		if ( i2c.Read(i2c_address, i2c_buffer, len) != len )
		{
			error_code = i2c.getErrorNumber();
		}
	}

	// Write packet: Start + Address(W) + N Bytes + Stop
	private void i2c_write_packet(int len)
	{
		error_code = 0;
		if ( i2c.Write(i2c_address, i2c_buffer, len) != len )
			error_code = i2c.getErrorNumber();
	}

	private void write_command(int cmd_code)
	{
		int j = 0;
		i2c_buffer[j++] = cmd_code;

		i2c_write_packet(j);
	}

	protected int read_word_register(int cmd_code)
	{
		int ret_val;

		i2c_buffer[0] = cmd_code;
		i2c_read_packet(2);

		ret_val = (i2c_buffer[0] & 0xff);
		ret_val <<= 8;
		ret_val |= (i2c_buffer[1] & 0xff);

		return ret_val;
	}

	protected void write_word_register(int cmd_code, int val)
	{
		int j = 0;
		i2c_buffer[j++] = cmd_code;
		i2c_buffer[j++] = (val >> 8) & 0xff;
		i2c_buffer[j++] = val & 0xff;

		i2c_write_packet(j);
	}

	protected int read_byte_register(int cmd_code)
	{
		i2c_buffer[0] = cmd_code;

		i2c_read_packet(1);

		return i2c_buffer[0] & 0xff;
	}

	protected void write_byte_register(int cmd_code, int val)
	{
		int j = 0;
		i2c_buffer[j++] = cmd_code;
		i2c_buffer[j++] = val & 0xff;

		i2c_write_packet(j);
	}

	public void Init()
	{
		byte val = (1<<R1_BIT)|(0<<R0_BIT);

		if (one_shot_mode)
			val |= (1<<ONESHOT_BIT);

		WriteConfiguration(val);	//delay to write eeprom
	}

	public void Open(int port)
	{
		i2c.Open(port);
	}

	public void Close()
	{
		i2c.Close();
	}

	public int StartConversion()
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		if ( ds1631_mode )
			write_command( CmdStartConversionExt );
		else
			write_command(CmdStartConversion);

		return getError();
	}

	public int StopConversion()
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		write_command(CmdStopConversion);

		return getError();
	}

	public int ReadConfiguration(int val[])
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		val[0] = read_byte_register(CmdAccessConfig);

		return getError();
	}

	public int WriteConfiguration(int val)
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		write_byte_register(CmdAccessConfig, val);

		return getError();
	}
/**
int DS1621::ReadHighThreshold(short &val)
{
	val = read_sword_register(CmdAccessTH);

	return GetError();
}

int DS1621::WriteHighThreshold(short val)
{
	write_sword_register(CmdAccessTL, val);

	return GetError();
}

int DS1621::ReadLowThreshold(short &val)
{
	val = read_sword_register(CmdAccessTL);

	return GetError();
}

int DS1621::WriteLowThreshold(short val)
{
	write_sword_register(CmdAccessTL, val);

	return GetError();
}
**/

	public int ReadTemperature(int val[], int index)
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		val[index] = read_word_register(CmdReadTemp);
		if (getError() != 0)
			val[index] = 0;

		return getError();
	}

	public int ReadCounter(int val[], int index)
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		val[index] = read_byte_register(CmdReadCounter);
		if (getError() != 0)
			val[index] = 0;

		return getError();
	}

	public int ReadSlope(int val[], int index)
	{
		i2c.clearErrorNumber();
	    i2c.PowerPulse();               //Charge VCC capacitor
		val[index] = read_byte_register(CmdReadSlope);
		if (getError() != 0)
			val[index] = 0;

		return getError();
	}

	public int WaitConversion()
	{
		int val[] = new int[1];
		int rv = 0;

		i2c.clearErrorNumber();
		if (one_shot_mode)
		{
			int k;

			k = 20;
			do {
				val[0] = 0;
				rv = ReadConfiguration(val);
			//	System.out.print("Config: " + Integer.toHexString(val[0]) + "\r\n");
				k--;
			} while ( k > 0 && rv == 0 && (val[0] & (1<<DONE_BIT)) == 0 );

		    if (k <= 0)
				rv = E2P_TIMEOUT;
		}
		else
		{
			i2c.PowerPulse(500);

			val[0] = 0;
			rv = ReadConfiguration(val);
		//	System.out.print("Config: " + Integer.toHexString(val[0]) + "\r\n");
		}

		return rv;
	}
}